BOOTH_CONFIG = "BOOTH_CONFIG"
BOOTH_KEY = "BOOTH_KEY"
CIB = "CIB"
COROSYNC_AUTHKEY = "COROSYNC_AUTHKEY"
COROSYNC_CONF = "COROSYNC_CONF"
PACEMAKER_AUTHKEY = "PACEMAKER_AUTHKEY"
PCSD_ENVIRONMENT_CONFIG = "PCSD_ENVIRONMENT_CONFIG"
PCSD_SSL_CERT = "PCSD_SSL_CERT"
PCSD_SSL_KEY = "PCSD_SSL_KEY"
PCS_KNOWN_HOSTS = "PCS_KNOWN_HOSTS"
PCS_SETTINGS_CONF = "PCS_SETTINGS_CONF"
