<form method=post action="/manage/removecluster">
  <table cellpadding=0 cellspacing=0 style="width: 100%; padding-left: 5px;">
    <tr>
      <th>All</th>
      <th><div class="check sprites" title="OK"></div></th>
      <th><div class="warning sprites" title="Warning"></div></th>
      <th><div class="error sprites" title="Failed"></div></th>
      <th style="width: 20px;"><div class="x sprites" title="Unknown"></div></th>
    </tr>
    <tr>
      <td>{{ Pcs.clusterController.cluster_list.length }}</td>
      <td style="padding-left: 5px;">{{ Pcs.clusterController.num_ok }}</td>
      <td style="padding-left: 5px;">{{ Pcs.clusterController.num_warning }}</td>
      <td style="padding-left: 5px;">{{ Pcs.clusterController.num_error }}</td>
      <td style="padding-left: 5px;">{{ Pcs.clusterController.num_unknown }}</td>
    </tr>
  </table>
  <table cellpadding=0 cellspacing=0 id="clusters_list">
    <tr>
      {{#if Pcs.clusterController.cluster_list.length}}
      <th></th><th></th><th>NAME</th><th style="padding-right: 1em;">NODES</th><th>RESOURCES</th><th style="padding-right: 16px;"></th>
      {{/if}}
    </tr>
    {{#each Pcs.clusterController.cluster_list }}
    <tr onmouseover="hover_over(this);" onmouseout="hover_out(this);" onclick="Pcs.clusterController.update_cur_cluster($(this).attr('nodeID'));" {{bind-attr nodeID="this.name"}}>
    <td class="node_list_check">
      <input class="node_list_check" type="checkbox" {{bind-attr name="input_name"}} {{bind-attr res_id="name"}}>
    </td>
    <td>
      {{{status_icon}}}
    </td>
    <td nowrap class="resource_name">
      {{#if forbidden}}
        {{name}}
      {{else}}
        <a {{bind-attr href=url_link}}>{{name}}</a>
      {{/if}}
    </td>
    <td nowrap class="resource_type">
      {{#if forbidden}}
        unknown
      {{else}}
        {{nodes.length}}
        {{#if nodes_failed}}
        | <div style="display: inline-block;" title="Issue(s) found"><span style="font-weight: bold; color: red">{{nodes_failed}}</span></div>
        {{/if}}
      {{/if}}
    </td>
    <td nowrap class="resource_type">
      {{#if status_unknown}}
        unknown
      {{else}}
        {{resource_list.length}}
        {{#if resources_failed}}
      | <div style="display: inline-block;" title="Issue(s) found"><span style="font-weight: bold; color: red">{{resources_failed}}</span></div>
        {{/if}}
      {{/if}}
    </td>
    <td>
      <div style="display:none" class="arrow sprites"></div>
    </td>
    </tr>
    {{/each}}
  </table>
</form>
