require 'rubygems'
require 'test/unit'

require 'test_corosyncconf.rb'
require 'test_cluster.rb'
require 'test_cluster_entity.rb'
require 'test_auth.rb'
require 'test_permissions.rb'
require 'test_config.rb'
require 'test_cfgsync.rb'
require 'test_pcs.rb'
